/* Release v0.9 dd 8 november 2017 of software for sensorboard_MPU6050 with a MPU6050 
This software reads elevation data from MPU6050 and when enabled also azimuth data from
a 12 bits encoder MAB25 or HH-12 (see at the start of the program the integer: what_to_measure) 

New (tuned) offset values can be writen to the MPU6050 when the line: write_offsets() (in 'void startup') is enabled.

This program is compiled by ON4CDU but some ideas and code are taken from the internet;
specific from Kristian Lauszus and Jeff Rowberg

 This is GNU software: GPL3
 
Comments are welcome at ON4CDU(at)uba.be
 */

#include <Wire.h>

// MAKE YOUR CHOICE
int what_to_measure = 2; // 1 for elevation data only (MPU6050); 2 for both elevation (MPU6050) and azimuth (MAB25 or HH-12)

float accX, accY, accZ;
long tempx,tempy,tempz;
byte i2cData[12]; // Buffer for I2C data
float el;
unsigned int code;
int rxtx_control = 3; // pin for controlling MAX485
unsigned int k = 0;

// offsets for MPU6050
int xoff = -2515; //-2458;
int yoff = 1744; //1744;
int zoff = 1834;

// MAB25 or HH-12 encoder conections (for azimuth values)
int clockPin = 7; // clock
int ChipSelect = 6; // Chip Select for encoder
int inputPin = 4; // encoderdata 

// variables for encoder data
int inputdata;
int statusdata = 0; // statusdata from encoder; will not be used in the program
int angledata = 0;  // angle data from encoder
int mask_lo_byte = 0xFF; // 000011111111; mask for getting least significant byte
int shortdelay = 1; // clock delay

//frame
unsigned char del = 0xD1;      // delimiter frame; D1 el only; D2 az and el frames
unsigned char AZ_hi_byte = 0; // azimuth high byte value
unsigned char AZ_lo_byte = 0; // azimuth low byte value
unsigned char EL_hi_byte = 0; // elevation high byte value
unsigned char EL_lo_byte = 0; // elevation low byte value
unsigned char endframe = 0x13;  // end of frame byte

byte IMUAddress = 0x68; // MP6050 address; AD0 is logic low on the PCB
unsigned int I2C_TIMEOUT = 1000; // Used to check for errors in I2C communication

void encoder_data() // routine to read the 12 bits encoder
{
  int inputstream = 0;
  angledata = 0;
  statusdata = 0;
  digitalWrite(ChipSelect, HIGH); // CS high for disabling encoder
  digitalWrite(clockPin, HIGH);   // CLK high
  delay(shortdelay);              // delaytime between readings
  digitalWrite(ChipSelect, LOW);  // CS low for enabling encoder, start transfer
  delayMicroseconds(shortdelay);  // delay for chip initialization
  digitalWrite(clockPin, LOW);    // start clocking
  delayMicroseconds(shortdelay);  // hold low

  for (int i = 0; i < 12; i++) // clock signal, 12 transitions for reading angle data
  {
    digitalWrite(clockPin, HIGH);        // clock goes high
    delayMicroseconds(shortdelay);       // delaytime between readings
    inputstream = digitalRead(inputPin); // read one bit of data from pin
    angledata = ((angledata << 1) + inputstream);// left shift and add pin value
    digitalWrite(clockPin, LOW);
    delayMicroseconds(shortdelay);      // end of one clock cycle
  }
  for (int i = 0; i < 8; i++) // clock signal, 8 transitions for reading status data
  {
    digitalWrite(clockPin, HIGH);  // clock goes high
    delayMicroseconds(shortdelay); //
    inputstream = digitalRead(inputPin); // read one bit of data from pin
    statusdata = ((statusdata << 1) + inputstream);// left shift and add pin value
    digitalWrite(clockPin, LOW);
    delayMicroseconds(shortdelay); // end of one clock cycle
  }
  digitalWrite(ChipSelect, HIGH); // CS high
}

void decode_az_data() // routine to transform the az data to bytes
{
  AZ_hi_byte = (angledata) >> 8; // shift 8 bits to form az high byte
  AZ_lo_byte = angledata & mask_lo_byte;
}

void sendframe() // routine to send data via RS485
{
  int i;
  char buf [6] = {del, EL_hi_byte, EL_lo_byte, AZ_hi_byte, AZ_lo_byte, endframe};

  for (i = 0; i < 6; i++)
  {
    digitalWrite (13, HIGH); // led on
    digitalWrite (rxtx_control, HIGH);
    Serial.write(buf[i]);
    delay(2);
    digitalWrite (rxtx_control, LOW);
    digitalWrite (13, LOW); // led off
  }
}

void write_offsets()  // routine for writing new offset values to mems
{  
  i2cData[0] = xoff>>8;
  i2cData[1] = xoff&0xFF;
  i2cData[2] = yoff>>8;
  i2cData[3] = yoff&0xFF;
  i2cData[4] = zoff>>8;
  i2cData[5] = zoff&0xFF;
  while (i2cWrite(0x06, i2cData, 6, false));
}

void read_acc_data() // routine to read acc data from mems
{
 while (i2cRead(0x3B, i2cData, 12));
  accX = ((i2cData[0] << 8) | i2cData[1]);
  accY = ((i2cData[2] << 8) | i2cData[3]);
  accZ = ((i2cData[4] << 8) | i2cData[5]); 
}

byte i2cWrite(byte registerAddress, byte data, boolean sendStop) {
  return i2cWrite(registerAddress, &data, 1, sendStop); // Returns 0 on success
}

byte i2cWrite(byte registerAddress, byte *data, byte length, boolean sendStop) {
  Wire.beginTransmission(IMUAddress);
  Wire.write(registerAddress);
  Wire.write(data, length);
  byte rcode = Wire.endTransmission(sendStop); // Returns 0 on success
  if (rcode) {
    Serial.println("i2cWrite failed: ");
    Serial.println(rcode);
  }
  return rcode; // See: http://arduino.cc/en/Reference/WireEndTransmission
}

byte i2cRead(byte registerAddress, byte *data, byte nbytes) {
  unsigned long timeOutTimer;
  Wire.beginTransmission(IMUAddress);
  Wire.write(registerAddress);
  byte rcode = Wire.endTransmission(false); // Don't release the bus
  if (rcode) {
    Serial.println("i2cRead failed: ");
    Serial.println(rcode);
    return rcode; // See: http://arduino.cc/en/Reference/WireEndTransmission
  }

  Wire.requestFrom(IMUAddress, nbytes, (byte)true); // Send a repeated start and then release the bus after reading
  for (byte i = 0; i < nbytes; i++) {
    if (Wire.available())
      data[i] = Wire.read();
    else {
      timeOutTimer = micros();
      while (((micros() - timeOutTimer) < I2C_TIMEOUT) && !Wire.available());
      if (Wire.available())
        data[i] = Wire.read();
      else {
        Serial.println("i2cRead timeout");
        return 5; // This error value is not already taken by endTransmission
      }
    }
  }
  return 0; // Success
}

void setup() {
  Serial.begin(9600); //(115200);
  Serial.println("software release 0.9"); 
  Wire.begin();
  pinMode(clockPin, OUTPUT); // clock
  pinMode(ChipSelect, OUTPUT); // Chip Select az encoder
  pinMode(inputPin, INPUT);   // input encoderdata (AZ & EL)
  pinMode(rxtx_control, OUTPUT); // direction control rs485
  i2cData[0] = 0x07; // Set the sample rate to 1 KHz
  i2cData[1] = 0x06; // Disable FSYNC and set 5 Hz Acc filtering, 5 Hz Gyro filtering, 1 KHz sampling
  i2cData[2] = 0x00; // Set Gyro Full Scale Range to ±250deg/s; not needed for this application
  i2cData[3] = 0x00; // Set Accelerometer Full Scale Range to ±2g
  while (i2cWrite(0x19, i2cData, 4, false)); // Write to all four registers at once
  while (i2cWrite(0x6B, 0x00, true)); // disable sleep mode and use internal oscillator; not sure if this statement is nessessary

  while (i2cRead(0x75, i2cData, 1)); // Read "WHO_AM_I" register
  if (i2cData[0] != 0x68) { 
    Serial.println("Error reading sensor");
    while (1);
  }

//  write_offsets(); // ENABLE THIS WHEN YOU WANT TO WRITE NEW OFFSETS TO THE MPU6050

  delay(100); // Wait for elevation sensor to stabilize
}

void loop() {
  
for (int j=0; j<50; j++){
  read_acc_data(); // Update all acc values 

// simple low pass filter
  tempx = tempx - (tempx>>7) + accX;
  accX = (tempx>>7);
  tempy = tempy - (tempy>>7) + accY;
  accY = (tempy>>7);
//tempz = tempz - (tempy>>7) + accZ;
//accZ = (tempz>>7); 
}

if ((k%2) == 1){

  if ((accX > 0) && (accY > 0)){
    el = (atan(accX/accY) *RAD_TO_DEG);
  }
 
  if ((accX > 0) && (accY < 0)){
    el = (atan(accY/accX) *RAD_TO_DEG);
    el = 90 - el;
  } 

   if ((accX < 0) && (accY < 0)){
    el = (atan(accX/accY) *RAD_TO_DEG);
     el = el + 180;
  }

   if ((accX < 0) && (accY > 0)){
    el = (atan(accY/accX) *RAD_TO_DEG);
    el = 270 - el;
  }

    code = int ((el * 4096)/360);

    EL_hi_byte = code >> 8; // low byte shifted out                      
    EL_lo_byte = (code & 0x00FF);  //mask  for getting LSB

  }
if (what_to_measure==2){
    del = 0xD2;
    encoder_data();
    decode_az_data();
  }   
    
    sendframe();
    k++;
    delay(20); 
}


  
